--[[
	Copyright (C) Udorn (Blackhand)
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.	
--]]

--[[
	Calculates statistics about all auctions seen.
--]]
vendor.Statistics = vendor.Vendor:NewModule("Statistics", "AceHook-3.0")
local L = vendor.Locale.GetInstance()

local log = vendor.Debug:new("Statistics")

--[[
	Sets the given value in the profile.
--]]
local function _SetValue(info, value)
	self.db.profile[info.arg] = value
end

--[[
	Returns the given value from the profile.
--]]
local function _GetValue(info)
	return self.db.profile[info.arg]
end

--[[
	Starts a new scan.
--]]
local function _StartScan(self, name, minLevel, maxLevel, invTypeIndex, classIndex, subclassIndex, page, isUsable, qualityIndex, getAll)
	log:Debug("_StartScan")
	if (self.scan) then
		wipe(self.scan.index)
	else
		self.scan = {}
		self.scan.index = {}
	end
	self.scan.name = name
	self.scan.minLevel = minLevel
	self.scan.maxLevel = maxLevel
	self.scan.invTypeIndex = invTypeIndex
	self.scan.classIndex = classIndex
	self.scan.subclassIndex = subclassIndex
	self.scan.isUsable = isUsable
	self.scan.qualityIndex = qualityIndex
	self.scan.getAll = getAll
	self.scan.length = 0
	self.scan.curPage = 0
	self.scan.completed = nil
end

--[[
	Checks whether a new scan has started.
--]]
local function _CheckScan(self, name, minLevel, maxLevel, invTypeIndex, classIndex, subclassIndex, page, isUsable, qualityIndex, getAll)
	log:Debug("_CheckScan")
	if (not self.scan) then
		_StartScan(self, name, minLevel, maxLevel, invTypeIndex, classIndex, subclassIndex, page, isUsable, qualityIndex, getAll)
	else
		local scan = self.scan
		if (scan.name ~= name or scan.minLevel ~= minLevel or scan.maxLevel ~= maxLevel or
			scan.invTypeIndex ~= invTypeIndex or scan.classIndex ~= classIndex or
		scan.subclassIndex ~= subclassIndex or scan.isUsable ~= isUsable or scan.qualityIndex ~= qualityIndex) then
			_StartScan(self, name, minLevel, maxLevel, invTypeIndex, classIndex, subclassIndex, page, isUsable, qualityIndex, getAll)
		else
			self.scan.curPage = page
		end
	end
end

--[[
	Extracts the acuctions from the snapshot corresponding to the current scan.
	Returns a list of the auctions extracted.
--]]
local function _ExtractOldAuctions(self)
	local snapshot
	if (vendor.AuctionHouse:IsNeutral()) then
		snapshot = self.db.realm.snapshot
	else
		snapshot = self.db.factionrealm.snapshot
	end
end

--[[
	Remove all auctions in the given table, which are also contained
	in the current scan.
--]]
local function _RemoveDuplicates(self, oldAuctions)
end

--[[
	Reinsert the auction of the given table to the snapshot.
--]]
local function _ReinsertAuctions(self, oldAuctions)
end

--[[
	Insert the auction of the current scan to the snapshot.
--]]
local function _InsertScan(self)
end

--[[
	Remembers the auctions found in the current scan.
--]]
local function _RememberScan(self, complete)
	local oldAuctions = _ExtractOldAuctions(self)
	_RemoveDuplicates(self, oldAuctions)
	if (not complete) then
		-- readd the old auctions not matched
		_ReinsertAuctions(self, oldAuctions)
	end
	_InsertScan(self)
end

--[[
	Initializes the module.
--]]
function vendor.Statistics:OnInitialize()
	self.db = vendor.Vendor.db:RegisterNamespace("Statistics", {
		profile = {
    	},
    	factionrealm = {
			snapshot = {},
		},
		realm = {
			snapshot = {}		
		},
	})
end

--[[
	Initializes the module.
--]]
function vendor.Statistics:OnEnable()
	self.canUseEval = vendor.CanUseEval:new()
	self:RawHook("GetAuctionItemInfo", true)
	self:SecureHook("QueryAuctionItems")
end

--[[
	Hooks the retrieval of auction infos, for being able to build statistics.
--]]
function vendor.Statistics:GetAuctionItemInfo(ahType, index)
	local name, texture, count, quality, canUse, level, minBid,
			minIncrement, buyoutPrice, bidAmount, highBidder, owner,
			saleStatus = self.hooks["GetAuctionItemInfo"](ahType, index)
			
--	local itemLink = GetAuctionItemLink(ahType, index)
--	if (itemLink) then
--    	if (self.canUseEval:IsUsable(itemLink)) then
--    		owner = "T("..(owner or "")..")"
--    	else
--    		owner = "F("..(owner or "")..")"
--    	end
--	end

	if (name and self.scan and not self.scan.completed and ahType == "list") then
		local idx = self.scan.curPage.."-"..index
		if (not self.scan.index[idx]) then
			log:Debug("GetAuctionItemInfo new item: %s", name)
			local itemLink = GetAuctionItemLink(ahType, index)
			local itemLinkKey = vendor.Items:GetItemLinkKey(itemLink)
			self.scan.index[idx] = strjoin(":", itemLinkKey, name, count, quality, level or "", minBid,
			minIncrement or "", buyoutPrice or "", bidAmount or "", highBidder or "", owner or "", saleStatus or "", GetTime())
			self.scan.length = self.scan.length + 1
			local batch, count = GetNumAuctionItems(ahType)
			log:Debug("GetAuctionItemInfo count: %d length: %d", count, self.scan.length)
			if (self.scan.length >= count) then
				log:Debug("GetAuctionItemInfo reached maximal number of auctions: %d", count)
				_RememberScan(self, true)
				self.scan.completed = true
    		end
    	else
    		log:Debug("GetAuctionItemInfo already known: %s", idx)
		end 
	end
	return name, texture, count, quality, canUse, level, minBid,
			minIncrement, buyoutPrice, bidAmount, highBidder, owner, saleStatus
end

--[[
	Hooks the Query call to be able to recognize scans.
--]]
function vendor.Statistics:QueryAuctionItems(name, minLevel, maxLevel, invTypeIndex, 
  classIndex, subclassIndex, page, isUsable, qualityIndex, getAll)
	_CheckScan(self, name, minLevel, maxLevel, invTypeIndex, classIndex, subclassIndex, page, isUsable, qualityIndex, getAll) 
end